package config

// singleFileSupport lists servers that can work without a project root.
var singleFileSupport = map[string]struct{}{
	"asm_lsp":      {},
	"bashls":       {},
	"clangd":       {},
	"cmake":        {},
	"cssls":        {},
	"dockerls":     {},
	"gopls":        {},
	"html":         {},
	"intelephense": {},
	"jsonls":       {},
	"lua_ls":       {},
	"marksman":     {},
	"nil_ls":       {},
	"nixd":         {},
	"pylsp":        {},
	"pyright":      {},
	"ruby_lsp":     {},
	"ruff":         {},
	"solargraph":   {},
	"taplo":        {},
	"texlab":       {},
	"ts_ls":        {},
	"vimls":        {},
	"yamlls":       {},
	"zls":          {},
}

// snippetSupport lists servers that support snippets.
var snippetSupport = map[string]struct{}{
	"ansiblels":              {},
	"asm_lsp":                {},
	"astro":                  {},
	"bashls":                 {},
	"buf_ls":                 {},
	"clangd":                 {},
	"clojure_lsp":            {},
	"cmake":                  {},
	"cssls":                  {},
	"dartls":                 {},
	"dockerls":               {},
	"elixirls":               {},
	"erlangls":               {},
	"fsautocomplete":         {},
	"golangci_lint_ls":       {},
	"gopls":                  {},
	"graphql":                {},
	"hls":                    {},
	"html":                   {},
	"intelephense":           {},
	"java_language_server":   {},
	"jsonls":                 {},
	"julials":                {},
	"kotlin_language_server": {},
	"lua_ls":                 {},
	"marksman":               {},
	"metals":                 {},
	"nil_ls":                 {},
	"nixd":                   {},
	"ocamllsp":               {},
	"perlnavigator":          {},
	"prismals":               {},
	"pylsp":                  {},
	"pyright":                {},
	"r_language_server":      {},
	"ruby_lsp":               {},
	"ruff":                   {},
	"rust_analyzer":          {},
	"solargraph":             {},
	"solidity_ls":            {},
	"sourcekit":              {},
	"svelte":                 {},
	"tailwindcss":            {},
	"taplo":                  {},
	"terraformls":            {},
	"texlab":                 {},
	"ts_ls":                  {},
	"vimls":                  {},
	"vue_ls":                 {},
	"yamlls":                 {},
	"zls":                    {},
}
