package pony

// Border styles.
const (
	BorderNone    = "none"
	BorderNormal  = "normal"
	BorderRounded = "rounded"
	BorderThick   = "thick"
	BorderDouble  = "double"
	BorderHidden  = "hidden"
)

// Alignment constants.
const (
	AlignmentLeading  = "leading"  // Horizontal: left side
	AlignmentCenter   = "center"   // Horizontal and vertical: center
	AlignmentTrailing = "trailing" // Horizontal: right side
	AlignmentTop      = "top"      // Vertical: top
	AlignmentBottom   = "bottom"   // Vertical: bottom
)

// Size constraint units.
const (
	UnitAuto    = "auto"
	UnitMin     = "min"
	UnitMax     = "max"
	UnitPercent = "%"
)

// Underline styles (matching UV).
const (
	UnderlineNone   = "none"
	UnderlineSingle = "single"
	UnderlineDouble = "double"
	UnderlineCurly  = "curly"
	UnderlineDotted = "dotted"
	UnderlineDashed = "dashed"
	UnderlineSolid  = "solid"
)

// Text decoration styles.
const (
	DecorationUnderline     = "underline"
	DecorationStrikethrough = "strikethrough"
)

// Font weight values.
const (
	FontWeightBold = "bold"
)

// Font style values.
const (
	FontStyleItalic = "italic"
)
